from dataclasses import dataclass
from typing import Callable, Optional, Sequence, Any
import logging

logger = logging.getLogger('janet')


@dataclass
class Device:
    identifiers: Sequence[str]
    available: bool

    name: str
    manufacturer: Optional[str] = None
    model: Optional[str] = None
    sw_version: Optional[str] = None
    kind: Optional[str] = None


class Channel:
    def __init__(self):
        self._listeners = set()

    def put(self, source: Device, entity: Any) -> None:
        remove = set()
        for l in self._listeners:
            try:
                if l(source, entity):
                    remove.add(l)
            except Exception as ex:
                logger.info(f'Listener raised {ex}, removing')
                remove.add(l)
                raise
        self._listeners ^= remove

    def listen(self, listener: Callable[[Device, Any], None]) -> None:
        self._listeners.add(listener)
