import time
import threading

from . import Device
from . import prometheus
from . import mqtt
from . import ssdp
from . import bravia

import janet.heyo.protocol
import janet.heyo.schema

import prometheus_client
import serial


def _heyo(clients):
    host = 'socket://192.168.1.4:9876'
    port = serial.serial_for_url(host)
    device = Device(identifiers=[f'heyo_{host}'],
                    kind='heyo',
                    name='heyo',
                    available=True)

    def poker():
        while True:
            port.write(janet.heyo.protocol.framer(3, b'\x00\x00\x00\x3e'))
            time.sleep(10)

    threading.Thread(target=poker, daemon=True).start()

    for frame in janet.heyo.protocol.deframer(port):
        for client in clients:
            client(device, frame)
        msg = janet.heyo.schema.decode(frame.payload)
        if msg:
            for client in clients:
                client(device, msg)


def main():
    prometheus_client.start_http_server(9874)
    prom = prometheus.Client()
    mq = mqtt.Client('192.168.1.4')
    clients = (prom.publish, mq.publish)

    threading.Thread(target=_heyo, args=(clients, )).start()

    s = ssdp.Discoverer()
    for client in clients:
        s.listen(client)

    b = bravia.Discoverer()
    s.listen(b.ssdp)
    for client in clients:
        b.listen(client)

    threading.Thread(target=s.run).start()

    while True:
        time.sleep(1)


if __name__ == '__main__':
    main()
